@@ -20,6 +20,7 @@ rootdir = libdir.dirname
 $:.unshift libdir
 
 # Monkeypatches
+require 'schleuder/mail/parts_list.rb'
 require 'schleuder/mail/message.rb'
 require 'schleuder/gpgme/import_status.rb'
 require 'schleuder/gpgme/key.rb'
@@ -37,7 +38,9 @@ require 'schleuder/version'
 require 'schleuder/logger_notifications'
 require 'schleuder/logger'
 require 'schleuder/listlogger'
-require 'schleuder/plugins_runner'
+require 'schleuder/plugin_runners/base'
+require 'schleuder/plugin_runners/list_plugins_runner'
+require 'schleuder/plugin_runners/request_plugins_runner'
 Dir["#{libdir}/schleuder/plugins/*.rb"].each do |file|
   require file
 end
@@ -62,9 +65,6 @@ ENV["SCHLEUDER_ROOT"] = rootdir.to_s
 GPGME::Ctx.set_gpg_path_from_env
 GPGME::Ctx.check_gpg_version
 
-# Require lib for database specified in config.
-require Schleuder::Conf.database['adapter']
-
 # TODO: Test if database is writable if sqlite.
 ActiveRecord::Base.establish_connection(Schleuder::Conf.database)
 ActiveRecord::Base.logger = Schleuder.logger
