@@ -40,7 +40,7 @@ module Schleuder
     end
 
     def self.list_keys(arguments, list, mail)
-      args = arguments.presence
+      args = Array(arguments.presence || '')
       args.map do |argument|
         # In this case it shall be allowed to match keys by arbitrary
         # sub-strings, therefore we use `list.gpg` directly to not have the
@@ -76,6 +76,9 @@ module Schleuder
       end
     end
 
+    # helper methods
+    private
+
     def self.is_armored_key?(material)
       return false unless /^-----BEGIN PGP PUBLIC KEY BLOCK-----$/ =~ material
       return false unless /^-----END PGP PUBLIC KEY BLOCK-----$/ =~ material
